//----------------------------------------------------------------------------
//
//		This file is part of the FALCON II.
//
//		The FALCON II is free software: you can redistribute it and/or modify
//		it under the terms of the GNU General Public License as published by
//		the Free Software Foundation, either version 3 of the License, or
//		(at your option) any later version.
//
//		The FALCON II is distributed in the hope that it will be useful,
//		but WITHOUT ANY WARRANTY; without even the implied warranty of
//		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//		GNU General Public License for more details.
//
//		You should have received a copy of the GNU General Public License
//		along with the FALCON II.  If not, see http://www.gnu.org/licenses/.
//
//		The FALCON II is copyright 2008-2009 by Timothy Pearson
//		tpearson@raptorengineeringinc.com
//		http://www.raptorengineeringinc.com
//
//----------------------------------------------------------------------------
//
//		The FALCON II is available as a reference design for the 
//		Raptor Engineering VDFPGA series of FPGA development boards
//		Please visit http://www.raptorengineeringinc.com for more information.
//
//----------------------------------------------------------------------------

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <unistd.h>
#include <fcntl.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "connection.h"
#include "gtk_image_auxfunctions.h"
#include "gtk_common_tasks.h"

#include "main.h"

extern unsigned char terminate_now;

GtkWidget *loginwindow;
char LoginOKClicked;
char LoginCancelClicked;

extern GtkWidget *ipchangewindow;
char IPChangeOKClicked;
char IPChangeCancelClicked;

extern GtkWidget *currentwidget;

extern unsigned char ip_q1;
extern unsigned char ip_q2;
extern unsigned char ip_q3;
extern unsigned char ip_q4;

extern unsigned char nm_q1;
extern unsigned char nm_q2;
extern unsigned char nm_q3;
extern unsigned char nm_q4;

extern unsigned char gw_q1;
extern unsigned char gw_q2;
extern unsigned char gw_q3;
extern unsigned char gw_q4;

extern unsigned char frame_requested;
extern unsigned char frame_requested_request;
extern int x;
extern int y;

extern unsigned int rx_buffer_count;

extern int data_goahead;
extern unsigned char sending_frame;
extern int disable_connection_alive_timer;
extern unsigned char want_to_grab_something[5];

extern GdkPixbuf *imagebuffer;
extern GdkPixbuf *hiddenbuffer;
extern GdkPixbuf *dctbuffer;
extern GdkPixbuf *uploadimage_buffer;

extern char filename_rnd[4096];

unsigned int ReadModelData (unsigned int *results) {
  GtkWidget *nmonitor;
  GtkTextBuffer *textbuffer;
  GtkTextIter start;
  GtkTextIter end;
  gchar *modeltext_substring;

  unsigned int i;
  unsigned int x;
  unsigned int read_pointer_offset;
  unsigned int string_length;
  gchar modeltext_currentnumber[1025];
  gchar modeltext_contents[100010];

  //nmonitor = lookup_widget (main_window, "dcttext");
  //textbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (nmonitor));

  //gtk_text_buffer_get_start_iter(textbuffer, &start);
  //gtk_text_buffer_get_end_iter(textbuffer, &end);
  x = gtk_text_iter_get_offset(&start);
  x = gtk_text_iter_get_offset(&end);
  read_pointer_offset = 0;
  //strncpy(modeltext_contents, gtk_text_buffer_get_text(textbuffer, &start, &end, FALSE), 100000);
  string_length = strlen(modeltext_contents);
  if (string_length > 100000) {
	string_length = 100000;
  }
  i=0;
  results[i]=0;		// Adjust for the unknown offset problem in models > 0
  i++;
  while (read_pointer_offset < string_length) {
  	modeltext_substring = strstr(modeltext_contents + read_pointer_offset, ",");
	if (modeltext_substring != NULL) {
		if ((modeltext_substring - modeltext_contents + read_pointer_offset) < 1024) {
			strncpy(modeltext_currentnumber, modeltext_contents + read_pointer_offset, modeltext_substring - (modeltext_contents + read_pointer_offset));
		}
		else {
			strncpy(modeltext_currentnumber, modeltext_contents + read_pointer_offset, 1024);
		}
		modeltext_currentnumber[(modeltext_substring - (modeltext_contents + read_pointer_offset))] = 0;
		results[i] = atoi(modeltext_currentnumber);
		read_pointer_offset = read_pointer_offset + (1 + (modeltext_substring - (modeltext_contents + read_pointer_offset)));
		i++;
	}
	else {
		// Terminate
		read_pointer_offset = string_length;
	}
  }

  return i;
}

void DisplayResults (void) {
  // Get the result information and put it in the caption
  GtkWidget *nmonitor;
  GtkWidget *imagebox;
  GtkImage *dctbox;
  GtkTextBuffer *textbuffer;

  unsigned char temp_buffer[1024];
  unsigned char temp_buffer2[10000];

  unsigned int mycolor;
  unsigned int currenterror;
  unsigned char red;
  unsigned char green;
  unsigned char blue;
  unsigned char alpha;

  int i;
  int j;
  int k;
  int x;
  int y;
  int x_write;
  int y_write;

  y = 240;
  j=0;
  temp_buffer2[0] = 0;
  for (x=319;x>0;x--) {
	mycolor = get_pixel(hiddenbuffer, x, y);
	mycolor = ((mycolor & 0xffffff00) >> 8) | ((mycolor & 0x000000ff) << 24);

	blue = mycolor & 0xff;
	green = (mycolor & 0xff00) >> 8;
	red = (mycolor & 0xff0000) >> 16;
	alpha = (mycolor & 0xff000000) >> 24;
	mycolor = (((green << 8) | red) << 16) | (((alpha << 8) | blue) << 0);
	currenterror = mycolor;

	x--;

	mycolor = get_pixel(hiddenbuffer, x, y);
	mycolor = ((mycolor & 0xffffff00) >> 8) | ((mycolor & 0x000000ff) << 24);

	red = mycolor & 0xff;
	green = (mycolor & 0xff00) >> 8;
	blue = (mycolor & 0xff0000) >> 16;
	alpha = (mycolor & 0xff000000) >> 24;

	itoa(currenterror, temp_buffer);
	strcat(temp_buffer2, temp_buffer);
	strcat(temp_buffer2, ": ");
	itoa(mycolor, temp_buffer);
	strcat(temp_buffer2, temp_buffer);
	strcat(temp_buffer2, ", ");

	j++;
	if (j>2) {
		strcat(temp_buffer2, "\n");
		j=0;
	}
  }

  currentwidget = lookup_widget (main_window, "resultstext");
  //gtk_label_set_text (currentwidget, temp_buffer2);
}

void DisplayDCTValue (unsigned int desired_dct_number) {
  // Get the DCT information for the desired DCT number and put it in the textbox
  GtkWidget *nmonitor;
  GtkWidget *imagebox;
  GtkImage *dctbox;
  GtkTextBuffer *textbuffer;

  unsigned char temp_buffer[1024];
  unsigned char temp_buffer2[1024];

  unsigned int mycolor;
  unsigned int dctone;
  unsigned int dcttwo;
  unsigned char red;
  unsigned char green;
  unsigned char blue;
  unsigned char alpha;

  int i;
  int j;
  int k;
  int x;
  int y;
  int z;
  int x_write;
  int y_write;

  int myaddress;
  int dct_number;
  int rppc_number;
  int dct_read_offset;

  unsigned char dct_assembly_toggler;
  unsigned int dct_temp_reg;
  unsigned int dct_assembler;

#define DCT_START_PIXEL_X 640
#define DCT_START_PIXEL_Y 240
#define DCT_PORTION_WIDTH 320
#define DCT_FULLIMAGE_WIDTH 640
#define DCT_SINGLE_LENGTH 512
#define DCT_SINGLE_WIDTH 32
#define RPPC_SINGLE_LENGTH 1024

  //nmonitor = lookup_widget (main_window, "dcttext");
  //textbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (nmonitor));

  //gtk_text_buffer_set_text (textbuffer, "", -1);

  x=DCT_START_PIXEL_X-1;
  y=DCT_START_PIXEL_Y;

  dct_number = desired_dct_number;
  rppc_number = 0;
  dct_read_offset = 0;

  x_write = 0;
  y_write = 0;
  k = 0;
  dct_assembly_toggler = 0;

  for (x=0;x<160;x++) {
	for (y=0;y<32;y++) {
		put_pixel(dctbuffer, x, y, 0, 0, 0, 0);
	}
  }

  	myaddress = (((DCT_FULLIMAGE_WIDTH/2) * (DCT_START_PIXEL_Y+1)) - (dct_number * 3 * ((DCT_SINGLE_LENGTH/2) + ((2*16)/2))) - (rppc_number * RPPC_SINGLE_LENGTH) - (dct_read_offset));
	myaddress = myaddress - 0;
	y=0;
	while (myaddress >= (DCT_FULLIMAGE_WIDTH/2)) {
		y++;
		myaddress = myaddress - (DCT_FULLIMAGE_WIDTH/2);
	}
	x = myaddress + 320;

  z=1;
  for (i=0;i<3;i++) {
	for (dct_read_offset=0;dct_read_offset<(DCT_SINGLE_LENGTH/2);dct_read_offset++) {
		x--;
		if (x < 320) {
			x=639;
			y--;
		}
	
		mycolor = get_pixel(hiddenbuffer, x, y);
		mycolor = ((mycolor & 0xffffff00) >> 8) | ((mycolor & 0x000000ff) << 24);
	
		blue = mycolor & 0xff;
		green = (mycolor & 0xff00) >> 8;
		red = (mycolor & 0xff0000) >> 16;
		alpha = (mycolor & 0xff000000) >> 24;
	
		// Ordering is now correct--dctone is higher bits, and comes before dcttwo in RAM
		dctone = (green << 8) | red;
		dcttwo = (alpha << 8) | blue;
	
		switch (dct_assembly_toggler) {
			case 0:
				if (dctone < 1024) {
					dct_assembler = dctone << 20;
				}
				else {
					dct_assembler = 0x3FF << 20;
				}
				if (dcttwo < 1024) {
					dct_assembler = (dct_assembler | (dcttwo << 10));
				}
				else {
					dct_assembler = (dct_assembler | (0x3FF << 10));
				}
				dct_assembly_toggler = 1;
				break;
			case 1:
				if (dctone < 1024) {
					dct_assembler = (dct_assembler | (dctone << 0));
				}
				else {
					dct_assembler = (dct_assembler | (0x3FF << 0));
				}
				dct_temp_reg = dcttwo;
	
				itoa(dct_assembler, temp_buffer2);
				//gtk_text_buffer_insert_at_cursor(textbuffer, temp_buffer2, -1);
				//gtk_text_buffer_insert_at_cursor(textbuffer, ",\t", -1);
				z++;
	
				dct_assembly_toggler = 2;
				break;
			case 2:
				if (dct_temp_reg < 1024) {
					dct_assembler = dct_temp_reg << 20;
				}
				else {
					dct_assembler = 0x3FF << 20;
				}
				if (dctone < 1024) {
					dct_assembler = (dct_assembler | (dctone << 10));
				}
				else {
					dct_assembler = (dct_assembler | (0x3FF << 10));
				}
				if (dcttwo < 1024) {
					dct_assembler = (dct_assembler | (dcttwo << 0));
				}
				else {
					dct_assembler = (dct_assembler | (0x3FF << 0));
				}
				
				itoa(dct_assembler, temp_buffer2);
//				gtk_text_buffer_insert_at_cursor(textbuffer, temp_buffer2, -1);
//				gtk_text_buffer_insert_at_cursor(textbuffer, ",\t", -1);
				z++;
	
				dct_assembly_toggler = 0;
				break;
		}
	
		dctone = dctone >> 2;
		dcttwo = dcttwo >> 2;
		if (dctone > 255) {
			dctone = 255;
		}
		if (dcttwo > 255) {
			dcttwo = 255;
		}
	
		put_pixel(dctbuffer, ((31-x_write) + k), (31-y_write), 0, dctone, 0, 255);
		x_write++;
		put_pixel(dctbuffer, ((31-x_write) + k), (31-y_write), 0, dcttwo, 0, 255);
		x_write++;
		if (x_write > 31) {
			x_write=0;
			y_write++;
			// Skip over pixel 33 and 34
			x--;
			if (x < 320) {
				x=639;
				y--;
			}
		}
		if (y_write > 15) {
			y_write=0;
			k=k+40;
		}
	}
  }

  // Mirror the bottom halves of the DFTs to the upper half
  for (k=0;k<120;k=k+40) {
	for (x=0;x<32;x++) {
		for (y=0;y<16;y++) {
			mycolor = get_pixel(dctbuffer, x+k, 31-y);
			blue = mycolor & 0xff;
			green = (mycolor & 0xff00) >> 8;
			red = (mycolor & 0xff0000) >> 16;
			alpha = (mycolor & 0xff000000) >> 24;
			put_pixel(dctbuffer, x+k, y, red, green, blue, 255);
		}
	}
  }

  dctbox = lookup_widget (main_window, "dctimage");
  gtk_image_set_from_pixbuf(dctbox, dctbuffer);
}

gboolean
quit                                   (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	stop_connection();
	terminate_now = 1;
	gtk_main_quit();
	return FALSE;
}


void
on_nconnect_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *nmonitor;
	char *myerrortxt;

	gtk_widget_set_sensitive(GTK_WIDGET(button), 1);
	nmonitor = lookup_widget (main_window, "nmonitor");
	gtk_widget_set_sensitive(nmonitor, 0);
	nmonitor = lookup_widget (main_window, "nconnect");
	gtk_widget_set_sensitive(nmonitor, 0);
	DisableMainCommands();
	gtk_main_iteration();

	myerrortxt = start_connection();

	if (myerrortxt[0] != 0) {
		MsgBox(myerrortxt);

CancelConnect:
		stop_connection();

		gtk_widget_set_sensitive(GTK_WIDGET(button), 1);
		nmonitor = lookup_widget (main_window, "nmonitor");
		gtk_widget_set_sensitive(nmonitor, 0);
		nmonitor = lookup_widget (main_window, "nconnect");
		gtk_widget_set_sensitive(nmonitor, 1);
		DisableMainCommands();
	}
	else {
		// Connect succeeded!
		gtk_widget_set_sensitive(GTK_WIDGET(button), 0);
		nmonitor = lookup_widget (main_window, "nmonitor");
		gtk_widget_set_sensitive(nmonitor, 1);
		EnableMainCommands();
		nmonitor = lookup_widget (main_window, "getdctsresultsbutton");
		gtk_widget_set_sensitive(nmonitor, 1);
		nmonitor = lookup_widget (main_window, "changeip");
		gtk_widget_set_sensitive(nmonitor, 1);
		nmonitor = lookup_widget (main_window, "nconnect");
		gtk_widget_set_sensitive(nmonitor, 0);
	}

}


void
on_nmonitor_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *nconnect;
	GtkWidget *nmonitor;

	gtk_widget_set_sensitive(GTK_WIDGET(button), 0);
	nconnect = lookup_widget (main_window, "nconnect");
	DisableMainCommands();
	nmonitor = lookup_widget (main_window, "changeip");
	gtk_widget_set_sensitive(nmonitor, 0);
	gtk_widget_set_sensitive(nconnect, 1);
	stop_connection();
}


void
on_sethv_clicked                       (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *text_value;
	char *textin;
	unsigned char textsend[64];
	int result;
	
	text_value = lookup_widget (main_window, "text_value");
	textin = gtk_entry_get_text(GTK_ENTRY(text_value));
	textsend[0] = 0;
	strcat(textsend, "HF ");
	strcat(textsend, textin);
	result = send_data(textsend);
}


void
on_setpower_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *text_value;
	char *textin;
	unsigned char textsend[64];
	int result;
	
	text_value = lookup_widget (main_window, "text_value");
	textin = gtk_entry_get_text(GTK_ENTRY(text_value));
	textsend[0] = 0;
	strcat(textsend, "HD ");
	strcat(textsend, textin);
	result = send_data(textsend);
}


void
on_loginbutton_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	LoginOKClicked=1;
}


void
on_cancelbutton_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	LoginCancelClicked=1;
}


void
on_sendcommand_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *text_value;
	char *textin;
	unsigned char textsend[64];
	int result;
	
	text_value = lookup_widget (main_window, "text_value");
	textin = gtk_entry_get_text(GTK_ENTRY(text_value));
	textsend[0] = 0;
	strcat(textsend, textin);
	result = send_data(textsend);
}


void
on_changeip_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	int result;
	unsigned char tempchar12[14];

	result = send_data("I2");
	
	// Show the IP change dialog
	IPChangeOKClicked=0;
	IPChangeCancelClicked=0;
	ipchangewindow = create_ipchange_window ();
	gtk_window_set_modal(GTK_WINDOW(ipchangewindow), TRUE);
	gtk_window_set_transient_for(GTK_WINDOW(ipchangewindow),GTK_WINDOW(main_window));
	gtk_widget_show (ipchangewindow);
	while ((IPChangeOKClicked==0)&&(IPChangeCancelClicked==0)) {
		gtk_main_iteration();
	}
	if (IPChangeCancelClicked==1) {
		// Do nothing
	}
	else {
		// Upload new IP address
		currentwidget = lookup_widget (ipchangewindow, "ip_dotquad_1");
		ip_q1 = atoi(gtk_entry_get_text (currentwidget));
		currentwidget = lookup_widget (ipchangewindow, "ip_dotquad_2");
		ip_q2 = atoi(gtk_entry_get_text (currentwidget));
		currentwidget = lookup_widget (ipchangewindow, "ip_dotquad_3");
		ip_q3 = atoi(gtk_entry_get_text (currentwidget));
		currentwidget = lookup_widget (ipchangewindow, "ip_dotquad_4");
		ip_q4 = atoi(gtk_entry_get_text (currentwidget));

		currentwidget = lookup_widget (ipchangewindow, "netmask_dotquad_1");
		nm_q1 = atoi(gtk_entry_get_text (currentwidget));
		currentwidget = lookup_widget (ipchangewindow, "netmask_dotquad_2");
		nm_q2 = atoi(gtk_entry_get_text (currentwidget));
		currentwidget = lookup_widget (ipchangewindow, "netmask_dotquad_3");
		nm_q3 = atoi(gtk_entry_get_text (currentwidget));
		currentwidget = lookup_widget (ipchangewindow, "netmask_dotquad_4");
		nm_q4 = atoi(gtk_entry_get_text (currentwidget));

		currentwidget = lookup_widget (ipchangewindow, "gateway_dotquad_1");
		gw_q1 = atoi(gtk_entry_get_text (currentwidget));
		currentwidget = lookup_widget (ipchangewindow, "gateway_dotquad_2");
		gw_q2 = atoi(gtk_entry_get_text (currentwidget));
		currentwidget = lookup_widget (ipchangewindow, "gateway_dotquad_3");
		gw_q3 = atoi(gtk_entry_get_text (currentwidget));
		currentwidget = lookup_widget (ipchangewindow, "gateway_dotquad_4");
		gw_q4 = atoi(gtk_entry_get_text (currentwidget));

		tempchar12[0] = 'I';
		tempchar12[1] = 'P';
		tempchar12[2] = ip_q1;
		tempchar12[3] = ip_q2;
		tempchar12[4] = ip_q3;
		tempchar12[5] = ip_q4;
		tempchar12[6] = nm_q1;
		tempchar12[7] = nm_q2;
		tempchar12[8] = nm_q3;
		tempchar12[9] = nm_q4;
		tempchar12[10] = gw_q1;
		tempchar12[11] = gw_q2;
		tempchar12[12] = gw_q3;
		tempchar12[13] = gw_q4;

		result = send_data(tempchar12);
	}
	gtk_object_destroy(ipchangewindow);
	// End showing the IP change dialog
}


void
on_thipch_cancel_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	IPChangeCancelClicked=1;
}


void
on_thipch_ok_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	IPChangeOKClicked=1;
}

void GrabAFrameNow (void) {
	GtkWidget *imagebox;
	GtkWidget *getframebutton;
	int result;
	
	DisableMainCommands();

	frame_requested_request = 1;
	x = 0;
	y = 0;

	rx_buffer_count = 0;

	// Send frame request
	want_to_grab_something[0] = 'G';
	want_to_grab_something[1] = 'R';
	want_to_grab_something[2] = '\n';
	want_to_grab_something[3] = '\r';
	want_to_grab_something[4] = 0;
}

void
on_getframebutton_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GrabAFrameNow();
}

gboolean
on_eventbox1_button_press_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkWidget *currentwidget;
  unsigned char temp_buffer[1024];
  unsigned char temp_buffer2[1024];
  unsigned int mycolor;
  unsigned char red;
  unsigned char green;
  unsigned char blue;
  unsigned char alpha;
  unsigned int desired_dct;
  signed int current_x;
  signed int current_y;

  if (event->type == GDK_BUTTON_PRESS) {
	currentwidget = lookup_widget (main_window, "imagecolor");
	mycolor = get_pixel(imagebuffer, event->x, event->y);
	red = mycolor & 0xff;
	green = (mycolor & 0xff00) >> 8;
	blue = (mycolor & 0xff0000) >> 16;
	alpha = (mycolor & 0xff000000) >> 24;
	temp_buffer[0]=0;
	strcat(temp_buffer, "R: ");
	itoa(red, temp_buffer2);
	strcat(temp_buffer, temp_buffer2);
	strcat(temp_buffer, " G: ");
	itoa(green, temp_buffer2);
	strcat(temp_buffer, temp_buffer2);
	strcat(temp_buffer, " B: ");
	itoa(blue, temp_buffer2);
	strcat(temp_buffer, temp_buffer2);
	strcat(temp_buffer, " Alpha: ");
	itoa(alpha, temp_buffer2);
	strcat(temp_buffer, temp_buffer2);
	//gtk_label_set_text (currentwidget, temp_buffer);
  }

  current_x = event->x;
  current_y = event->y;

  current_x = current_x - 320;
  if (current_x < 0) {
	current_x = 0;
  }
  current_y = current_y - 240;
  if (current_y < 0) {
	current_y = 0;
  }

  current_x = 320 - current_x;
  current_y = 240 - current_y;

  current_x = current_x - 16;
  current_y = current_y - 16;

  if (current_x < 0) {
	current_x = 0;
  }
  if (current_y < 0) {
	current_y = 0;
  }

  current_x = current_x / 32;
  current_y = current_y / 32;

  desired_dct = (current_y * ((320 / 32) - 1)) + current_x;

  if ((desired_dct * (((16 * 34 * 3) / 2))) > (320 * 240)) {
	MsgBox("Apologies, but that patch is not present in client memory\n\nI am giving you patch #0 instead");
	desired_dct = 0;
  }

  printf("I'm guessing you wanted DCT #%d?  If I'm wrong, I don't care! :-P\n", desired_dct);
  DisplayDCTValue(desired_dct);

  DisplayResults();
}


gboolean
on_eventbox1_enter_notify_event        (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
  GtkWidget *nmonitor;

  if (event->type == GDK_ENTER_NOTIFY) {
	nmonitor = lookup_widget (main_window, "getframebutton");
	gdk_window_set_cursor (nmonitor->window, gdk_cursor_new (GDK_CROSSHAIR));
  }
  else {
	nmonitor = lookup_widget (main_window, "getframebutton");
	gdk_window_set_cursor (nmonitor->window, gdk_cursor_new (GDK_LEFT_PTR));
  }

  return FALSE;
}


gboolean
on_eventbox1_leave_notify_event        (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
  GtkWidget *nmonitor;

  nmonitor = lookup_widget (main_window, "getframebutton");
  gdk_window_set_cursor (nmonitor->window, gdk_cursor_new (GDK_LEFT_PTR));

  return FALSE;
}

void
on_getdctsresultsbutton_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *imagebox;
	GtkWidget *getframebutton;
	GtkWidget *nmonitor;
	int result;
	
	DisableMainCommands();

	frame_requested_request = 2;
	x = 0;
	y = 0;

	rx_buffer_count = 0;

	// Send frame request for DCTs
	want_to_grab_something[0] = 'G';
	want_to_grab_something[1] = 'Q';
	want_to_grab_something[2] = '\n';
	want_to_grab_something[3] = '\r';
	want_to_grab_something[4] = 0;
}


void
on_loadmodel_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *nmonitor;
	GtkWidget *imagebox;
	unsigned int loadmodel_array[4096];
	unsigned int data_counter;
	int result;
	int i;
	int j;
	int k;
	int x;
	int y;
	char input_accepted;
	int model_number;
	char tempchar[4096];
	char datastring[64];
	char filename_tmp[4096];
	unsigned char send_array[1228800];
	//data_counter = ReadModelData(loadmodel_array);

	GtkWidget *file_dialog;
	file_dialog = gtk_file_chooser_dialog_new ("Select New Module File", main_window, GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
	FILE *fp;
	struct stat statb;
	int character_f;

	unsigned int mycolor;
	unsigned char red;
	unsigned char green;
	unsigned char blue;
	unsigned char alpha;

	GtkWidget *currentwidget;
	
	DisableMainCommands();

	// Load the image file
	input_accepted = 0;
	while (input_accepted == 0) {
		if (gtk_dialog_run (GTK_DIALOG (file_dialog)) == GTK_RESPONSE_ACCEPT) {
			char *filename;
			filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_dialog));

			// Validate the file
			if (stat(filename, &statb)<0) {
				statb.st_size = 0;
			}
			if (statb.st_size > 0) {
				// Use imagemagick to convert the file into
				// a format gdk can read natively
				random_pw(filename_rnd);
				sprintf(filename_tmp, "mkdir -p /tmp/falconclient_%s/", filename_rnd);
				system(filename_tmp);
				sprintf(filename_tmp, "rm -f /tmp/falconclient_%s/converted.png", filename_rnd);
				system(filename_tmp);
				sprintf(tempchar, "/usr/bin/convert %s -alpha activate -alpha reset /tmp/falconclient_%s/converted.png", filename, filename_rnd);
				system(tempchar);

				// Read the resultant file
				sprintf(filename_tmp, "/tmp/falconclient_%s/converted.png", filename_rnd);
				uploadimage_buffer = gdk_pixbuf_new_from_file_at_scale(filename_tmp, 320, 240, FALSE, NULL);
				if (uploadimage_buffer == NULL) {
					GtkWidget *errorbox;
					errorbox = gtk_message_dialog_new (main_window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "File is empty or cannot be read.\r\rCheck file permissions and try again.");
					gtk_dialog_run (GTK_DIALOG (errorbox));
					gtk_widget_destroy (errorbox);
				}
				else {
					input_accepted = 1;
				}
				sprintf(filename_tmp, "rm -rf /tmp/falconclient_%s", filename_rnd);
				system(filename_tmp);
			}
			else {
				GtkWidget *errorbox;
				errorbox = gtk_message_dialog_new (main_window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "File is empty or cannot be read.\r\rCheck file permissions and try again.");
				gtk_dialog_run (GTK_DIALOG (errorbox));
				gtk_widget_destroy (errorbox);
			}

			g_free (filename);
		}
		else {
			input_accepted = 2;
		}
	}
	gtk_widget_destroy (file_dialog);

	if (input_accepted == 1) {
		// Convert and load
		//imagebox = lookup_widget (main_window, "imagebox");
		//gtk_image_set_from_pixbuf(imagebox, uploadimage_buffer);

		printf("data_counter: %d\n", data_counter);
		printf("loadmodel_array[0]: %d\n", loadmodel_array[0]);
		printf("loadmodel_array[1]: %d\n", loadmodel_array[1]);
		printf("loadmodel_array[2]: %d\n", loadmodel_array[2]);
		
		nmonitor = lookup_widget (main_window, "modelslotnumber");
		model_number = gtk_spin_button_get_value(GTK_SPIN_BUTTON(nmonitor));
		
// 		// Load model command
// 		send_array[0] = 'L';
// 		send_array[1] = 'M';

		// Load destination address into FPGA L2
		sending_frame = 1;
		receive_data();
		data_goahead = 0;
		sprintf(datastring, "LA%d\n\r", 0);
		result = send_data(datastring);
		while (data_goahead == 0) {
			while (gtk_events_pending())
				gtk_main_iteration();
		}

		currentwidget = lookup_widget (main_window, "transfer_progress");
		gtk_progress_configure(currentwidget,1,0,9);

		j=0;
		x=320-1;
		y=240-1;
		for (k=0;k<8;k++) {
			for (i=0;i<(38400/4);i++) {
				// Copy the data into the send buffer
				x--;
				if (x < 0) {
					y--;
					x=320-1;
				}
				if (y>=0 && x>=0) {
					mycolor = get_pixel(uploadimage_buffer, x, y);
				}
			
				red = mycolor & 0xff;
				green = (mycolor & 0xff00) >> 8;
				blue = (mycolor & 0xff0000) >> 16;
				alpha = (mycolor & 0xff000000) >> 24;

				send_array[((i*4)+0)+2] = blue;
				send_array[((i*4)+1)+2] = alpha;
				send_array[((i*4)+2)+2] = red;
				send_array[((i*4)+3)+2] = green;
			}
			j=j+(i*4);
			// Load image command
			send_array[0] = 'L';
			send_array[1] = 'T';
			disable_connection_alive_timer = 1;
			data_goahead = 0;
			result = send_ndata(send_array, (38400 + 2));		// Image
			while (data_goahead == 0) {
				while (gtk_events_pending())
					gtk_main_iteration();
			}
			disable_connection_alive_timer = 0;
			currentwidget = lookup_widget (main_window, "transfer_progress");
			gtk_progress_configure(currentwidget,k+2,0,9);
			printf("\tChunk %d is away!\n\r", k);
		}

		printf("send_array[10]: %d\n", send_array[10]);
		printf("send_array[11]: %d\n", send_array[11]);
		printf("send_array[12]: %d\n", send_array[12]);

		// Destroy uploadimage_buffer to prevent memory leaks
		g_object_unref (G_OBJECT (uploadimage_buffer));

		// Update image
		result = send_data("CF\n\r");		// Update frame
		result = send_data("TR\n\r");		// Ask for status
		sending_frame = 0;
	}
	
	EnableMainCommands();
	GrabAFrameNow();
}


void
on_getframe_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  int result;

  result = send_data("CF\n\r");		// Image
}


void
on_setrgb_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
  int result;

  result = send_data("RG\n\r");		// RGB mode
}


void
on_setycrcb_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  int result;

  result = send_data("YC\n\r");		// YCrCb mode
}


void
on_startrecognition_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
  int result;

  result = send_data("BR\n\r");		// Start slow recognition
}


void
on_stoprecognition_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  int result;

  result = send_data("SR\n\r");		// Stop all recognition
}


void
on_new1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_save_as1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_cut1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_copy1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_paste1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_delete1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_l_switchbuffer_zero_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
  int result;
  char datastring[64];

  DisableMainCommands();
  sending_frame = 1;
  receive_data();
  data_goahead = 0;
  sprintf(datastring, "LB%d\n\r", 0);
  result = send_data(datastring);
  while (gtk_events_pending())
	gtk_main_iteration();
  sleep (1);
  while (data_goahead == 0) {
	while (gtk_events_pending())
		gtk_main_iteration();
  }
  sending_frame = 0;
  result = send_data("TR\n\r");		// Ask for status
  EnableMainCommands();
}


void
on_l_switchbuffer_one_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
  int result;
  char datastring[64];

  DisableMainCommands();
  sending_frame = 1;
  receive_data();
  data_goahead = 0;
  sprintf(datastring, "LB%d\n\r", 76800);
  result = send_data(datastring);
  while (gtk_events_pending())
	gtk_main_iteration();
  sleep (1);
  while (data_goahead == 0) {
	while (gtk_events_pending())
		gtk_main_iteration();
  }
  sending_frame = 0;
  result = send_data("TR\n\r");		// Ask for status
  EnableMainCommands();
}


void
on_l_switchbuffer_two_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
  int result;
  char datastring[64];

  DisableMainCommands();
  sending_frame = 1;
  receive_data();
  data_goahead = 0;
  sprintf(datastring, "LB%d\n\r", 153600);
  result = send_data(datastring);
  while (gtk_events_pending())
	gtk_main_iteration();
  sleep (1);
  while (data_goahead == 0) {
	while (gtk_events_pending())
		gtk_main_iteration();
  }
  sending_frame = 0;
  result = send_data("TR\n\r");		// Ask for status
  EnableMainCommands();
}


void
on_l_switchbuffer_custom_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
  int result;
  int newaddr;
  char datastring[64];
  GtkWidget *nmonitor;

  DisableMainCommands();
  nmonitor = lookup_widget (main_window, "l_switchbuffer_address");
  newaddr = gtk_spin_button_get_value(GTK_SPIN_BUTTON(nmonitor));
  sending_frame = 1;
  receive_data();
  data_goahead = 0;
  sprintf(datastring, "LB%d\n\r", newaddr);
  result = send_data(datastring);
  while (gtk_events_pending())
	gtk_main_iteration();
  sleep (1);
  while (data_goahead == 0) {
	while (gtk_events_pending())
		gtk_main_iteration();
  }
  sending_frame = 0;
  result = send_data("TR\n\r");		// Ask for status
  EnableMainCommands();
}


void
on_r_switchbuffer_zero_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
  int result;
  char datastring[64];

  DisableMainCommands();
  sending_frame = 1;
  receive_data();
  data_goahead = 0;
  sprintf(datastring, "RB%d\n\r", 0);
  result = send_data(datastring);
  while (gtk_events_pending())
	gtk_main_iteration();
  sleep (1);
  while (data_goahead == 0) {
	while (gtk_events_pending())
		gtk_main_iteration();
  }
  sending_frame = 0;
  result = send_data("TR\n\r");		// Ask for status
  EnableMainCommands();
}


void
on_r_switchbuffer_one_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
  int result;
  char datastring[64];

  DisableMainCommands();
  sending_frame = 1;
  receive_data();
  data_goahead = 0;
  sprintf(datastring, "RB%d\n\r", 76800);
  result = send_data(datastring);
  while (gtk_events_pending())
	gtk_main_iteration();
  sleep (1);
  while (data_goahead == 0) {
	while (gtk_events_pending())
		gtk_main_iteration();
  }
  sending_frame = 0;
  result = send_data("TR\n\r");		// Ask for status
  EnableMainCommands();
}


void
on_r_switchbuffer_two_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
  int result;
  char datastring[64];

  DisableMainCommands();
  sending_frame = 1;
  receive_data();
  data_goahead = 0;
  sprintf(datastring, "RB%d\n\r", 153600);
  result = send_data(datastring);
  while (gtk_events_pending())
	gtk_main_iteration();
  sleep (1);
  while (data_goahead == 0) {
	while (gtk_events_pending())
		gtk_main_iteration();
  }
  sending_frame = 0;
  result = send_data("TR\n\r");		// Ask for status
  EnableMainCommands();
}


void
on_r_switchbuffer_custom_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
  int result;
  int newaddr;
  char datastring[64];
  GtkWidget *nmonitor;

  DisableMainCommands();
  nmonitor = lookup_widget (main_window, "r_switchbuffer_address");
  newaddr = gtk_spin_button_get_value(GTK_SPIN_BUTTON(nmonitor));
  sending_frame = 1;
  receive_data();
  data_goahead = 0;
  sprintf(datastring, "RB%d\n\r", newaddr);
  result = send_data(datastring);
  while (gtk_events_pending())
	gtk_main_iteration();
  sleep (1);
  while (data_goahead == 0) {
	while (gtk_events_pending())
		gtk_main_iteration();
  }
  sending_frame = 0;
  result = send_data("TR\n\r");		// Ask for status
  EnableMainCommands();
}

gboolean
on_eventbox2_button_release_event      (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	extern GtkWidget *aboutwindow;
	extern GdkPixbuf *raptorlogo;
	GtkWidget *raptorlogobox;
	GtkWidget *nmonitor;

	nmonitor = lookup_widget (main_window, "getframebutton");
	aboutwindow = create_about_window ();
	gtk_window_set_modal(GTK_WINDOW(aboutwindow), TRUE);
	gtk_window_set_transient_for(GTK_WINDOW(aboutwindow),GTK_WINDOW(main_window));
	raptorlogobox = lookup_widget (aboutwindow, "raptorlogoboxii");
	gtk_image_set_from_pixbuf(raptorlogobox, raptorlogo);
	gtk_widget_show (aboutwindow);

	return FALSE;
}

gboolean
on_eventbox2_enter_notify_event        (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
  GtkWidget *nmonitor;

  if (event->type == GDK_ENTER_NOTIFY) {
	nmonitor = lookup_widget (main_window, "getframebutton");
	gdk_window_set_cursor (nmonitor->window, gdk_cursor_new (GDK_QUESTION_ARROW));
  }
  else {
	nmonitor = lookup_widget (main_window, "getframebutton");
	gdk_window_set_cursor (nmonitor->window, gdk_cursor_new (GDK_LEFT_PTR));
  }

  return FALSE;
}


gboolean
on_eventbox2_leave_notify_event        (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
  GtkWidget *nmonitor;

  nmonitor = lookup_widget (main_window, "getframebutton");
  gdk_window_set_cursor (nmonitor->window, gdk_cursor_new (GDK_LEFT_PTR));

  return FALSE;
}

