//----------------------------------------------------------------------------
//
//		This file is part of the FALCON II.
//
//		The FALCON II is free software: you can redistribute it and/or modify
//		it under the terms of the GNU General Public License as published by
//		the Free Software Foundation, either version 3 of the License, or
//		(at your option) any later version.
//
//		The FALCON II is distributed in the hope that it will be useful,
//		but WITHOUT ANY WARRANTY; without even the implied warranty of
//		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//		GNU General Public License for more details.
//
//		You should have received a copy of the GNU General Public License
//		along with the FALCON II.  If not, see http://www.gnu.org/licenses/.
//
//		The FALCON II is copyright 2008-2009 by Timothy Pearson
//		tpearson@raptorengineeringinc.com
//		http://www.raptorengineeringinc.com
//
//----------------------------------------------------------------------------
//
//		The FALCON II is available as a reference design for the 
//		Raptor Engineering VDFPGA series of FPGA development boards
//		Please visit http://www.raptorengineeringinc.com for more information.
//
//----------------------------------------------------------------------------

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"
#include "connection.h"
#include "raptorlogo_inline.c"

GtkWidget *main_window;
GtkWidget *loginwindow;
GtkWidget *ipchangewindow;
GtkWidget *aboutwindow;
GdkColor color;
GdkPixbuf *imagebuffer;
GdkPixbuf *hiddenbuffer;
GdkPixbuf *dctbuffer;
GdkPixbuf *raptorlogo;
GtkWidget *raptorlogobox;
GdkPixbuf *uploadimage_buffer;

char filename_rnd[4096];

char imagemagick_found;
char main_commands_status = 0;

int
main (int argc, char *argv[])
{

  gtk_set_locale ();
  gtk_init (&argc, &argv);
  GtkWidget *nmonitor;

  //add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");

  /*
   * The following code was added by Glade to create one of each component
   * (except popup menus), just so that you see something after building
   * the project. Delete any components that you don't want shown initially.
   */
  main_window = create_main_window ();
  gtk_widget_show (main_window);

  //raptorlogo = gdk_pixbuf_new_from_file("raptorlogo.png", NULL);
  raptorlogo = gdk_pixbuf_new_from_inline (-1, raptorlogo_inline, FALSE, NULL);
  raptorlogobox = lookup_widget (main_window, "raptorlogobox");
  gtk_image_set_from_pixbuf(raptorlogobox, raptorlogo);

  nmonitor = lookup_widget (main_window, "nmonitor");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "loadmodel");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "changeip");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "getframe");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "sendcommand");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "getframebutton");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "getdctsresultsbutton");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "setrgb");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "setycrcb");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "startrecognition");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "stoprecognition");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "l_switchbuffer_zero");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "l_switchbuffer_one");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "l_switchbuffer_two");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "l_switchbuffer_custom");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "r_switchbuffer_zero");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "r_switchbuffer_one");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "r_switchbuffer_two");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "r_switchbuffer_custom");
  gtk_widget_set_sensitive(nmonitor, 0);
  nmonitor = lookup_widget (main_window, "text_value");
  gtk_widget_set_sensitive(nmonitor, 0);

  // Verify that imagemagick is installed and warn if not!
  if (system("/usr/bin/convert > /dev/null") != 256) {
	// imagemagick not present!
	GtkWidget *errorbox;
	imagemagick_found = 0;
	errorbox = gtk_message_dialog_new (main_window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "ImageMagick tools are not installed on this system.\r\rPlease install imagemagick to activate image uploading functions.");
	gtk_dialog_run (GTK_DIALOG (errorbox));
	gtk_widget_destroy (errorbox);
  }
  else {
	imagemagick_found = 1;
  }

  g_timeout_add_full (G_PRIORITY_HIGH, 50, (GSourceFunc) receive_data, NULL, NULL);

  // Create the main image buffer
  imagebuffer = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480);

  // Create the hidden image buffer
  hiddenbuffer = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480);
  raptorlogobox = lookup_widget (main_window, "imagebox");
  gtk_image_set_from_pixbuf(raptorlogobox, hiddenbuffer);

  // Create the DCT image buffer
  dctbuffer = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 160, 32);

  gtk_main ();
  return 0;
}

